#--------------------------------------------------------------------
# GUI for combining and linearly interplolating data sets
# starting draft by A.McGhie - finished by B. Toby and R. Harlow

# queue a command to execute later
lappend command(ExecuteLater) SetupInterp

proc SetupInterp {} {
    global scriptdir interp menulist helplist
    # initial conditions
    if {$::tcl_platform(platform) == "windows"} {
	set interp(program) [file join $scriptdir linterp.exe]
    } else {
	set interp(program) [file join $scriptdir linterp]
    }
    
    if {[file exists $interp(program)]} {
	set interp(newname) {}
	foreach var {ttmin ttmax dlt} {
	    set interp($var) {}
	}
	lappend menulist(compute) Interp
	lappend menulist(pages) Interp
	set helplist(Interp) {
	    Interpolate File(s)
	}
    }

    foreach var {ttmin ttmax ns s} {
	trace variable interp($var) w UpdateInterpOutputFile
    }

    # deal with deleting regions
    bind .plot <Key-d> {SetupDeleteRegion %x}
    bind .plot <Key-D> {SetupDeleteRegion %x}
}

# create the Interp page (once)
proc MakeInterp {page} {
    global graph interp

    pack [frame $page.a] -side left -fill y -anchor n
    pack [label $page.a.t1 -text {Select Dataset(s) to}] -side top
    pack [label $page.a.t2 -text {perform an interpolation on}] -side top
    pack [frame $page.a.2 ] -side top 
    makeselectbox $page.a.2
    # makeselectbox creates $page.a.2.1 with datafiles
    bind $page.a.2.1 <<ListboxSelect>> "InterpUpdate $page.a.2"
    pack [frame $page.a1 -borderwidth 3 -relief groove]  -side left -fill y -anchor n
    pack [frame $page.a2 -borderwidth 3 -relief groove]  -side left -fill y -anchor n

    #################################################################

    #################################################################
    set rown 1

    #grid rowconfigure $page.a1 $rown -minsize 20
    #incr rown

    grid [label $page.a1.c1$rown -text Method] -column 1 -columnspan 2 -row $rown
    incr rown
    grid [radiobutton $page.a1.c1$rown -text "Linear interpolation" \
	    -variable interp(spline) -value 0 \
	    -command "DisableSpline $page.a.2"] \
	    -column 1 -columnspan 2 -row $rown -sticky w
    incr rown
    grid [radiobutton $page.a1.c1$rown -text "Spline interpolation" \
	    -variable interp(spline) -value 1 \
	    -command "DisableSpline $page.a.2"] \
	    -column 1 -columnspan 2 -row $rown -sticky w
    incr rown
    grid [radiobutton $page.a1.c1$rown -text "Rebin (no interpolation)" \
	    -variable interp(spline) -value 2 \
	    -command "DisableSpline $page.a.2"] \
	    -column 1 -columnspan 2 -row $rown -sticky w
    set interp(spline) 0

    incr rown
    grid rowconfigure $page.a1 $rown -minsize 20
    incr rown


    grid [label $page.a1.c1$rown -text {Variables}] -column 1 -columnspan 2 -row $rown

    incr rown
    grid [label $page.a1.c1$rown -text "Two theta min:  "] \
	    -column 1 -row $rown
 
    grid [entry $page.a1.c2$rown -width 8 \
	    -textvariable interp(ttmin)] -column 2 -row $rown 

    incr rown
    grid [label $page.a1.c1$rown -text "Two theta max:  "] -column 1 -row $rown 
    grid [entry $page.a1.c2$rown -width 8 \
	    -textvariable interp(ttmax)] -column 2 -row $rown 

    incr rown
    grid [label $page.a1.c1$rown -text "Two theta Delta:"] -column 1 -row $rown 
    grid [entry $page.a1.c2$rown -width 8 \
	    -textvariable interp(dlt)] -column 2 -row $rown

    incr rown
    grid rowconfigure $page.a1 $rown -minsize 20
    incr rown
    grid [label $page.a1.c1$rown -text {Spline Variables}] -column 1 -columnspan 2 -row $rown
    set  interp(spline_widgets) $page.a1.c1$rown

    incr rown
    grid [label $page.a1.c1$rown -text "Pts in spline:  "] -column 1 -row $rown
    lappend interp(spline_widgets) $page.a1.c1$rown
 
    grid [entry $page.a1.c2$rown -width 8 \
	    -textvariable interp(ns)] -column 2 -row $rown 
    lappend interp(spline_widgets) $page.a1.c2$rown 

    incr rown
    grid [label $page.a1.c1$rown -text "Spline smooth:  "] -column 1 -row $rown 
    lappend interp(spline_widgets) $page.a1.c1$rown
    grid [entry $page.a1.c2$rown -width 8 \
	    -textvariable interp(s)] -column 2 -row $rown 
    lappend interp(spline_widgets) $page.a1.c2$rown


    incr rown
    grid [label $page.a1.c1$rown -text {Output Name}] -column 1 -columnspan 2 -row $rown

    incr rown
    grid [entry $page.a1.c2$rown -width 25 \
	    -textvariable interp(newname)] -column 1 -columnspan 2 -row $rown

 
    #################################################################
    incr rown
    grid rowconfigure $page.a1 $rown -minsize 20
    incr rown
    grid [button $page.a1.b1 -text "Run" \
		-command "runInterp $page.a.2" ] -row $rown -column 1 -columnspan 3 
    set interp(run_button) $page.a1.b1

    pack [button $page.a.4    -text "  Update Plot  " \
	    	-command "getfilelist $page.a.2; displaylist"] -side bottom
    #################################################################

    grid [label $page.a2.b1x0 -text "Delete these regions"] -column 1 -row 0 -columnspan 3  
    grid [label $page.a2.b1x1 -text "No."] -column 1 -row 1 
    grid [label $page.a2.b1x2 -text "start"] -column 2 -row 1
    grid [label $page.a2.b1x3 -text "end"] -column 3 -row 1
    for {set rown 2 } {$rown <= 10} {incr rown} {
         grid [label $page.a2.b1$rown -text "[expr $rown -1]"] -column 1 -row $rown 
         grid [entry $page.a2.b2$rown -width 8 \
	    -textvariable interp(b$rown)] -column 2 -row $rown 
         grid [entry $page.a2.b3$rown -width 8 \
	    -textvariable interp(e$rown)] -column 3 -row $rown 	
#	 bind  $page.a2.b2$rown <FocusOut> {set interp(i) $rown}
#bind  $page.a2.b3$rown <Enter> {$page.a2.b3$rown config -bg red ;$entry config -show * }
    }	    
    DisableSpline $page.a.2
}

# "cleanup" the Interp page each time it is selected
proc PostPageInterp {page}  {
    global graph interp
    updateselectbox $page.a.2
    $interp(run_button) config -state disabled
    foreach w $interp(spline_widgets) {
	$w config -fg gray
	catch {$w config -state disabled}
    }
    for {set i 2} {$i <= 10} {incr i} {
	set interp(b$i) ""
	set interp(e$i) ""
    }
    set interp(deleteselected) 0
    eval $graph(blt) marker delete  [$graph(blt) marker names]
}    

# disable the spline inputs except when needed
proc DisableSpline {box} {
    InterpUpdate $box
    global interp
    foreach w $interp(spline_widgets) {
	if {$interp(spline) == 1} {
	    $w config -fg black
	    catch {$w config -state normal}
	} else {
		$w config -fg gray
	    catch {$w config -state disabled}
	}
    }
}

# update the data limits each time a file is selected or the 
# Interp method changes (see DisableSpline)
proc InterpUpdate {box} {
    global graph interp
    getfilelist $box
    set xmin ""
    set xmax ""
    set xstep ""
    if {[llength $graph(plotlist)] == 0} return
    foreach data $graph(plotlist) {
	global $data 
	if {[set ${data}(type)] == "peaks"} continue
	set xvector [set ${data}(xvector)]
	global $xvector
	set min [set ${xvector}(min)] 
	if {$xmin == ""} {
	    set xmin $min
	} elseif {$min < $xmin} {
	    set xmin $min
	}
	set max [set ${xvector}(max)]
	if {$xmax == ""} {
	    set xmax $max
	} elseif {$max > $xmax} {
	    set xmax $max
	}
	set step [expr {($max - $min)/([$xvector length] - 1)}]
	if {$xstep == ""} {
	    set xstep $step
	} elseif {$step > $xstep} {
	    set xstep $step
	}
    }
    set interp(ttmin) $xmin
    set interp(ttmax) $xmax
    if {$interp(spline) == 2} {
	set interp(dlt)   [expr {2*$xstep}]
    } else {
	set interp(dlt)   $xstep
    }
    if {[catch {expr {$xmin+$xmax+$xstep}}]} {
	$interp(run_button) config -state disable
    } else {
	$interp(run_button) config -state normal
    }
}

proc UpdateInterpOutputFile {args} {
   global interp graph
    set interp(newname) [lindex $graph(plotlist) end]
    append interp(newname) "_$interp(ttmin)"
    append interp(newname) "_$interp(ttmax)" 
    if {$interp(spline) == 1} {
	append interp(newname) "_$interp(ns)" 
    } elseif {$interp(spline) == 2} {
	append interp(newname) "_bin"
    }

}

proc SetupDeleteRegion { x } {
    global graph interp
    set x [format %.3f \
	    [$graph(blt) xaxis invtransform $x]]
    if {$interp(deleteselected) == 0} {
	for {set i 2} {$i <= 10} {incr i} {
	    if {[catch {expr {$interp(b$i)+$interp(e$i)}}]} {
		break
	    }
	}
	set interp(deleteselected) $i
	set interp(b$interp(deleteselected)) $x
    } else {
	set interp(e$interp(deleteselected)) $x
	set interp(deleteselected) 0
    }
    # redraw deleted regions
    eval $graph(blt) marker delete  [$graph(blt) marker names]
    if {$interp(deleteselected) == 0} {
	for {set i 2} {$i <= 10} {incr i} {
	    set x1 $interp(b$i)
	    set x2 $interp(e$i)

	    if {[catch {expr {$x1+$x2}}]} continue
	    $graph(blt) marker create line -coords "$x1 -Inf $x2 -Inf"\
		    $graph(MarkerColorOpt) yellow -linewidth 5 -under 1
	}
    }
}

proc runInterp {box} {
    global interp graph command
    getfilelist $box

    if {[llength $graph(plotlist)] == 0} return

    set dellist {}
    # list of deleted regions
    for {set i 2} {$i <= 10} {incr i} {
	set x1 $interp(b$i)
	set x2 $interp(e$i)
	if {[catch {expr {$x1+$x2}}]} continue
	lappend dellist [list $x1 $x2]
    }
    set dellist [lsort -index 0 -real $dellist]

    set fp [open interp.inp w]
    puts $fp "$interp(ttmin), $interp(ttmax), $interp(dlt)"
    puts $fp "$interp(spline)"
    if {$interp(spline) == 1} {
	puts $fp "$interp(ns), $interp(s)"
    }

#    puts $fp [llength $dellist]
#    foreach del $dellist {
#	puts $fp [join $del ", "]
#    }

    if {$interp(spline) >= 0} {
	foreach data $graph(plotlist) {
	    global $data 
	    if {[set ${data}(type)] == "peaks"} continue
	    set xvector [set ${data}(xvector)]
	    global $xvector
	    set yvector [set ${data}(yvector)]
	    global $yvector
	    set esdvec [set ${data}(esdvec)]
	    global $esdvec
	    foreach x [$xvector range 0 end] \
		    y [$yvector range 0 end] \
		    e [$esdvec  range 0 end] {
		puts $fp "$x, $y, $e"
	    }
	    puts $fp "-999., -999., -999."
	}
#    } else {
#	set bigdatalist {}
#	foreach data $graph(plotlist) {
#	    global $data 
#	    if {[set ${data}(type)] == "peaks"} continue
#	    set xvector [set ${data}(xvector)]
#	    global $xvector
#	    set yvector [set ${data}(yvector)]
#	    global $yvector
#	    set esdvec [set ${data}(esdvec)]
#	    global $esdvec
#	    foreach x [$xvector range 0 end] \
#		    y [$yvector range 0 end] \
#		    e [$esdvec  range 0 end] {
#		lappend bigdatalist [list $x $y $e]
#	    }
	}
#	foreach item [lsort -real -index 0 $bigdatalist] {
#	    puts $fp [join $item ", "]
	}
#    }
    close $fp

  #    run BT's fortran program
    exec $interp(program) < interp.inp
    source interp.out

    set newdata $interp(newname)

    initdata $newdata
    global ${newdata}
    # set attributes
    set ${newdata}(xlabel) "2theta"
    set ${newdata}(xunits) "2theta"
    set ${newdata}(ylabel) "intensity"
    # no skipped points
    set ${newdata}(skip) 0
    # store the results
    set ${newdata}(x) $newx
    set ${newdata}(y) $newy
    set ${newdata}(esd) $newesd
    # set rescaling to unity
    resetdata $newdata
    # add the new file to the plot list
    if {[lsearch $graph(plotlist) $newdata] == -1} {
	lappend graph(plotlist) $newdata
    }
    updateselectbox $box
    displaylist
    # file delete interp.inp
}

