# read ascii XY data files
#
# Modified to read x,y or x,y,esd and skip points where y < 0
# Will also read numbers separated by commas, and/or spaces
#
#                                                  BHT Oct 19, 1999
#
# From - Thu Oct 14 18:08:47 1999
# Date: Mon, 11 Oct 1999 17:40:59 -0400
# From: ajm <mcghieaj@esvax.es.dupont.com>
# To: "Brian H. Toby" <brian.toby@nist.gov>
# Subject: Another read file for you if you want it.
#
#Brian,
#	This read file for CMPR reads ascii pairs ( 2theta and intensity ). 
#I don't know if I handled the esd properly, but the program runs.
#
#It appears to be working for me. 
#
#----------------------------------------------------------------
#------- define a command line option -----------------------------
# command line option 
lappend command(cmdopt) -xyd
# proc to use 
set command(-xyd)  readXYdata
#--------define a dialog box entry
# menu label
lappend command(readtypes) "XY data (ascii)"
# proc for this entry
lappend command(readproc) ReadXYd
# allowed data types
if {$tcl_platform(platform) == "windows"} {
    lappend command(filterlist) {dat pks}
    # definitions for these data types
    set command(ReadXYd_dat_type) "XY data"
    set command(ReadXYd_pks_type) "XY peaks"
} else {
    lappend command(filterlist) {dat jones DAT JONES pks peaks PEAKS}
    # definitions for these data types
    set command(ReadXYd_dat_type) "XY data"
    set command(ReadXYd_DAT_type) "XY data"
    set command(ReadXYd_jones_type) "XY data"
    set command(ReadXYd_JONES_type) "XY data"
    set command(ReadXYd_pks_type) "XY peaks"
    set command(ReadXYd_peaks_type) "XY peaks"
    set command(ReadXYd_PEAKS_type) "XY peaks"
}

proc ReadXYd {file} {
    global command
    if {$file == ""} return
    pleasewait "reading file $file"
    set ret [readXYdata $file]
    donewait
    if {$ret != ""} {return $ret}
    showlastentry $command(read_filelist)

}
proc readXYdata {filename} {

    #puts "opening XY Data file $filename"
    global graph

    set lnum 0
    # signal errors by quitting
    if [catch {
	set fp [open $filename r]
	set data [file root [file tail $filename]] 
	# eliminate spaces from the name
	regsub -all " " $data "_" data
	global ${data}

	# Great!! Now we are positioned to read the data.

	if {[string index \
		[string tolower [file extension $filename]] 1] == "p"} {
	    initpeaks $data
	    gets $fp ${data}(title)
	    gets $fp wave
	    if {$wave == 0} {
		set ${data}(xlabel) "d-space"
		set ${data}(xunits) "dspace"
	    } elseif {$wave < 0} {
		set ${data}(xlabel) "Q"
		set ${data}(xunits) "Q"
	    } else {
		set ${data}(xlabel) "2theta"
		set ${data}(xunits) "2theta"
		set ${data}(wavelength) $wave
	    }
	    set ptsread 0
	    # Now, read the rest of the file.
	    # until we get to the last data point, an end-of-file or error
	    set i 0
	    while {[gets $fp line] >= 0} {
		incr lnum
		incr ptsread
		# change commas to blanks 
		regsub -all , $line " " line
		if {$lnum > 20000} {
		    return "Read too many lines. Something is wrong!"
		}
		catch {
		    set x 0
		    set x [expr [lindex $line 0]]
		}
		catch {
		    set y 0
		    set y [expr [lindex $line 1]]
		}
		lappend ${data}(x) $x
		lappend ${data}(y) $y
	    }
	    resetdata $data
	} else {
	    # Initialize the data structure.
	    initdata $data
	    set ${data}(xlabel) "2theta"
	    set ${data}(xunits) "2theta"
	    set ${data}(ylabel) "counts"
	    
	    # No missing points
	    set ${data}(skip) 0
	    set ptsread 0

	    # Now, read the rest of the file.
	    # until we get to the last data point, an end-of-file or error
	    set i 0
	    while {[gets $fp line] >= 0} {
		incr lnum
		incr ptsread
		# change commas to blanks 
		regsub -all , $line " " line
		if {$lnum > 20000} {
		    return "Read too many lines. Something is wrong!"
		}
		set yyyy -1
		catch {
		    set xxxx [expr [lindex $line 0]]
		    set yyyy [expr [lindex $line 1]]
		}
		set yyyyesd -1
		catch {
		    set yyyyesd [expr [lindex $line 2]]
		}
		# set the esd
		if {$yyyyesd < 0} {
		    if {$yyyy <= 0} {
			set yyyyesd 1
		    } else {
			set yyyyesd [expr sqrt(1.0 * $yyyy )] 
		    }
		}
		# skip negative or garbled points
		if {[lindex $line 1] >= 0 && $yyyy >= 0} {
		    lappend ${data}(x)   $xxxx
		    lappend ${data}(y)   $yyyy		
		    lappend ${data}(esd) $yyyyesd
		} else {
		    # there are skipped points
		    set ${data}(skip) 1
		}
		incr ptsread
	    }
	}
    } errmsg] {
	# try to process the data we have read
	catch {resetdata $data}
	return "XY Data error. Error reading line $lnum: $errmsg"
    }
    resetdata $data
    lappend graph(plotlist) $data
    catch {close $fp} 
    #puts "$ptsread valid points read"
    return {}
}
 
#-- 
#Alistair McGhie [DuPont Exp. Sta.  Wilm. De. 19880-0228]

